import axios from 'axios';
import { stringify } from 'qs';
import localStorage from '../storage/localStorage';
import cookie from '../storage/cookie';

let token = '';

export const handleToken = {
    setToken(Authorization) {
        token = Authorization;
        localStorage.set('Authorization', Authorization);
        cookie.set({ authorization: Authorization });
    },
    getToken() {
        if (token) {
            return token;
        } else if (localStorage.get('Authorization')) {
            return localStorage.get('Authorization');
        } else {
            return cookie.get('Authorization');
        }
    },
};

const formatContentType = function (contentType, data) {
    const map = {
        'application/x-www-form-urlencoded'(data) {
            return stringify(data);
        },
    };
    return (map[contentType] && map[contentType](data)) || data;
};

export const request = function (requestInfo) {
    const {
        path = '',
        method = 'get',
        baseURL = '',
        body = {},
        headers = {},
        query = {},
    } = requestInfo;

    if (baseURL && !baseURL.startsWith('http')) {
        throw new Error('set baseURL only support cross domain');
    }

    headers['Content-Type'] = headers['Content-Type'] || 'application/json';
    const req = axios({
        params: query,
        baseURL,
        method,
        url: path,
        data: formatContentType(headers['Content-Type'], body),
        headers,
        withCredentials: !baseURL,
    });
    return req
        .then((res = {}) => {
            const { data = {}, status, headers = {} } = res || {};
            if ((status + '').startsWith('2')) {
                const { Authorization, authorization } = headers;
                if (Authorization || authorization) {
                    handleToken.setToken(Authorization || authorization);
                }

                return data;
            }
            return Promise.reject({
                status,
                message: data.Message,
            });
        })
        .catch((err) => {
            // 处理code
            if (err === 'expired request') {
                throw err;
            }

            if (err.request) {
                const { responseText, status } = err.request;
                let message;
                try {
                    message = JSON.parse(responseText).Message;
                } catch (error) {
                    message = responseText;
                }
                err = {
                    status,
                    message,
                };
            }

            throw err;
        });
};

export default request;
